/**
 * App debugging methods
 * 
 * @author Brad Beebe
 * @since 1.0
 */	
Core.Helpers.Debug = {
	_bIsDebugMode: true,
	_tmrMemory: null,
	_aTimers: [],
	_aDebugLevels: [1, 2, 3, 4],
	_mapDebugLevels: {
		info: 1,
		debug: 2,
		warn: 3,
		error: 4
	},


	/**
	 * Returns whether or not the app is in debug mode
	 * 
	 * @return boolean
	 * 
	 * @author Brad Beebe
	 * @since 1.0
	 */
	isDebugMode: function(){
		return this._bIsDebugMode;
	},


	/**
	 * Sets the debug levels
	 * 
	 * @param array aLevels - An array of debug levels
	 * @return void
	 * 
	 * @author Brad Beebe
	 * @since 1.0
	 */
	setDebugLevels: function(aLevels){
		if(aLevels instanceof Array && aLevels.length > 0){
			this._aDebugLevels = aLevels;
		}
	},


	/**
	 * Logs an info message
	 * 
	 * @param string sMsg - Message to log
	 * @param number iNewLines - Number of new lines to insert before message (optional)
	 * @return void
	 * 
	 * @author Brad Beebe
	 * @since 1.0
	 */	
	info: function(sMsg, iNewLines){
		this.log('info', sMsg, iNewLines);
	},
	
	
	/**
	 * Logs a debugging message
	 * 
	 * @param string sMsg - Message to log
	 * @param number iNewLines - Number of new lines to insert before message (optional)
	 * @return void
	 * 
	 * @author Brad Beebe
	 * @since 1.0
	 */	
	debug: function(sMsg, iNewLines){
		this.log('debug', sMsg, iNewLines);
	},
	
	
	/**
	 * Logs a warning message
	 * 
	 * @param string sMsg - Message to log
	 * @param number iNewLines - Number of new lines to insert before message (optional)
	 * @return void
	 * 
	 * @author Brad Beebe
	 * @since 1.0
	 */	
	warn: function(sMsg, iNewLines){
		this.log('warn', sMsg, iNewLines);
	},
	
	
	/**
	 * Logs an error message
	 * 
	 * @param string sMsg - Message to log
	 * @param number iNewLines - Number of new lines to insert before message (optional)
	 * @return void
	 * 
	 * @author Brad Beebe
	 * @since 1.0
	 */	
	error: function(sMsg, iNewLines){
		this.log('error', sMsg, iNewLines);
	},
	
	
	/**
	 * Logs a message only if debug mode is set
	 * 
	 * @param string sLevel - Severity level
	 * @param string sMsg - Message to log
	 * @param number iNewLines - Number of new lines to insert before message (optional - defaults to zero)
	 * @return void
	 * 
	 * @author Brad Beebe
	 * @since 1.0
	 */
	log: function(sLevel, sMsg, iNewLines){		
		if(this._bIsDebugMode === true && this._aDebugLevels.indexOf(this._mapDebugLevels[sLevel]) !== -1){
			// Insert any new lines
			if(typeof iNewLines === 'number' && iNewLines > 0){
				for(var i = 0; i < iNewLines; i++){
					Ti.API.log(sLevel, ' ');
				}
			}
			
			Ti.API.log(sLevel, sMsg);
		}
	},
	
	
	/**
	 * Extracts the specific error message from a XHR error object's "error" property
	 * 
	 * @link http://developer.appcelerator.com/question/120192/httpclient-error-event-properties
	 * @param string sErr - XHR error object "error" property
	 * @return string
	 * 
	 * @author Brad Beebe
	 * @since 1.0
	 */
	getXhrError: function(sErr){
		var aMatch = sErr.match(/\"([^\"]+)\"/),
		sMsg = sErr.indexOf("ASIHTTPRequestError") > -1 && aMatch.length > 0 ? aMatch[1] : 'General XHR error';
			
		return sMsg;
	},
	
	
	/**
	 * Logs the XHR error message
	 * 
	 * @param object oArgs - Arguements object
	 * (
	 * 	@param object err - XHR onerror-passed error object
	 *	@param string prefix - String to prepend to the XHR error message
	 * )
	 * @return void
	 * 
	 * @author Brad Beebe
	 * @since 1.0
	 */
	logXhrError: function(oArgs){
		var sMsg = typeof oArgs.prefix === 'string' ? oArgs.prefix + ': ' : '';
		sMsg += this.getXhrError(oArgs.err.error);
			
		this.error(sMsg, 1);
	},
	
	
	/**
	 * Logs the current available memory every specified interval
	 * 
	 * @param number iMs - The interval duration in milliseconds
	 * @return void
	 * 
	 * @author Brad Beebe
	 * @since 1.0
	 */
	memMontiorStart: function(iMs){
		iMs = typeof iMs == 'number' && iMs > 0 ? iMs : 1000;
		
		this._tmrMemory = setInterval(function(){
			// We use the native warn() method instead of Debug.warn() since
			// practically we are only going to use this method while debugging
			// via the Titanium console thus no need to check for "debug" mode
			Ti.API.warn('Available memory: ' + Ti.Platform.availableMemory);
		}, iMs);
	},
	

	/**
	 * Stops the memory logging interval
	 * 
	 * @return void
	 * 
	 * @author Brad Beebe
	 * @since 1.0
	 */
	memMonitorEnd: function(){
		clearTimeout(this._tmrMemory);
	},
	
	
	/**
	 * Start a timer
	 * 
	 * @param string sLabel - Label (optional)
	 * @return void
	 * 
	 * @author Eric Shelkie
	 * @since 1.0
	 */	
	timerStart: function(sLabel){
		if(this._bIsDebugMode === true){
			aTimers[sLabel] = new Date().getTime();
		}
	},
	

	/**
	 * End a timer and log to console
	 * 
	 * @param string sLabel - Label (optional)
	 * @param string sSuffix - A note to include with output. eg. "since click"
	 * @return integer - Elapsed time in milliseconds
	 * 
	 * @author Eric Shelkie
	 * @since 1.0
	 */
	timerEnd: function(sLabel, sSuffix){
		var iElapsed = null;
		
		if(this._bIsDebugMode === true){
			var iStart;
			
			if(!sLabel){
				iStart = aTimers[aTimers.length - 1];
			}else if(typeof aTimers[sLabel] !== 'undefined'){
				iStart = aTimers[sLabel];
			}
			
			if(iStart){
				iElapsed = new Date().getTime() - iStart;
				
				if(!sSuffix){
					sSuffix = ((sLabel) ? 'since ' + sLabel + ' started' : '');
				}
				
				this.warn(iElapsed.toString() + 'ms ' + sSuffix, 1);
			}else{
				Core.Debug.warn('Debug.timerEnd(' + sLabel + ') called, but could not find a corresponding start time.');
			}
		}
		
		return iElapsed;
	}
};

// Alias the debugger helper
var Debug = Core.Helpers.Debug;